@@ -6,17 +6,25 @@
 
 #include "core/fxcrt/xml/cfx_xmlchardata.h"
 
+#include "core/fxcrt/xml/cfx_xmldocument.h"
 #include "third_party/base/ptr_util.h"
 
 CFX_XMLCharData::CFX_XMLCharData(const WideString& wsCData)
     : CFX_XMLText(wsCData) {}
 
-CFX_XMLCharData::~CFX_XMLCharData() {}
+CFX_XMLCharData::~CFX_XMLCharData() = default;
 
 FX_XMLNODETYPE CFX_XMLCharData::GetType() const {
   return FX_XMLNODE_CharData;
 }
 
-std::unique_ptr<CFX_XMLNode> CFX_XMLCharData::Clone() {
-  return pdfium::MakeUnique<CFX_XMLCharData>(GetText());
+CFX_XMLNode* CFX_XMLCharData::Clone(CFX_XMLDocument* doc) {
+  return doc->CreateNode<CFX_XMLCharData>(GetText());
+}
+
+void CFX_XMLCharData::Save(
+    const RetainPtr<IFX_SeekableWriteStream>& pXMLStream) {
+  pXMLStream->WriteString("<![CDATA[");
+  pXMLStream->WriteString(GetText().UTF8Encode().AsStringView());
+  pXMLStream->WriteString("]]>");
 }
