@@ -89,7 +89,8 @@ class CompObj
 {
 public:
 	//! the constructor
-	CompObj() : m_mapCls()
+	CompObj()
+		: m_mapCls()
 	{
 		initCLSMap();
 	}
@@ -97,7 +98,7 @@ public:
 	/** return the CLS Name corresponding to an identifier */
 	char const *getCLSName(unsigned long v)
 	{
-		if (m_mapCls.find(v) == m_mapCls.end()) return 0L;
+		if (m_mapCls.find(v) == m_mapCls.end()) return nullptr;
 		return m_mapCls[v];
 	}
 
@@ -221,15 +222,49 @@ protected:
 /** Internal: internal method to keep ole definition */
 struct OleDef
 {
-	OleDef() : m_id(-1), m_subId(-1), m_dir(""), m_name("") { }
-	int m_id /**main id*/, m_subId /**subsversion id */ ;
-	std::string m_dir/**the directory*/, m_name/**the base name*/;
+	OleDef()
+		: m_id(-1)
+		, m_base("")
+		, m_dir("")
+		, m_name("") { }
+	int m_id /**main id*/;
+	std::string m_base/** the base name*/, m_dir/**the directory*/, m_name/**the complete name*/;
+};
+
+/** Internal: the state of a WPSOLEParser */
+struct State
+{
+	//! constructor
+	State(libwps_tools_win::Font::Type fontType, std::function<int(std::string const &)> const &dirToIdFunc)
+		: m_fontType(fontType)
+		, m_directoryToIdFunction(dirToIdFunc)
+		, m_metaData()
+		, m_unknownOLEs()
+		, m_idToObjectMap()
+		, m_compObjIdName()
+	{
+	}
+	//! the default font type
+	libwps_tools_win::Font::Type m_fontType;
+	//! the function used to convert a directory name in a id
+	std::function<int(std::string const &)> m_directoryToIdFunction;
+	//! the meta data
+	librevenge::RVNGPropertyList m_metaData;
+	//! list of ole which can not be parsed
+	std::vector<std::string> m_unknownOLEs;
+	//! map id to object
+	std::map<int, WPSEmbeddedObject> m_idToObjectMap;
+
+	//! a smart ptr used to stored the list of compobj id->name
+	std::shared_ptr<WPSOLEParserInternal::CompObj> m_compObjIdName;
 };
 }
 
 // constructor/destructor
-WPSOLEParser::WPSOLEParser(const std::string &mainName)
-	: m_avoidOLE(mainName), m_unknownOLEs(), m_objects(), m_objectsId(), m_compObjIdName()
+WPSOLEParser::WPSOLEParser(const std::string &mainName, libwps_tools_win::Font::Type fontType,
+                           std::function<int(std::string const &)> const &dirToIdFunc)
+	: m_avoidOLE(mainName)
+	, m_state(new WPSOLEParserInternal::State(fontType, dirToIdFunc))
 {
 }
 
@@ -237,24 +272,65 @@ WPSOLEParser::~WPSOLEParser()
 {
 }
 
+
+std::vector<std::string> const &WPSOLEParser::getNotParse() const
+{
+	return m_state->m_unknownOLEs;
+}
+//! returns the list of data positions which have been read
+std::map<int,WPSEmbeddedObject> const &WPSOLEParser::getObjectsMap() const
+{
+	return m_state->m_idToObjectMap;
+}
+void WPSOLEParser::updateMetaData(librevenge::RVNGPropertyList &metaData) const
+{
+	librevenge::RVNGPropertyList::Iter i(m_state->m_metaData);
+	for (i.rewind(); i.next();)
+	{
+		if (!metaData[i.key()])
+			metaData.insert(i.key(),i()->clone());
+	}
+}
 // parsing
+int WPSOLEParser::getIdFromDirectory(std::string const &dirName)
+{
+	// try to retrieve the identificator stored in the directory
+	//  MatOST/MatadorObject1/ -> 1, -1
+	//  Object 2/ -> 2, -1
+	auto dir=dirName+'/';
+	auto pos = dir.find('/');
+	while (pos != std::string::npos)
+	{
+		if (pos >= 1 && dir[pos-1] >= '0' && dir[pos-1] <= '9')
+		{
+			auto idP = pos-1;
+			while (idP >=1 && dir[idP-1] >= '0' && dir[idP-1] <= '9')
+				idP--;
+			int val = std::atoi(dir.substr(idP, idP-pos).c_str());
+			return val;
+		}
+		pos = dir.find('/', pos+1);
+	}
+	WPS_DEBUG_MSG(("WPSOLEParser::getIdFromDirectory: can not find id for %s\n", dirName.c_str()));
+	return -1;
+}
+
 bool WPSOLEParser::parse(RVNGInputStreamPtr file)
 {
-	if (!m_compObjIdName)
-		m_compObjIdName.reset(new WPSOLEParserInternal::CompObj);
+	if (!m_state->m_compObjIdName)
+		m_state->m_compObjIdName.reset(new WPSOLEParserInternal::CompObj);
 
-	m_unknownOLEs.resize(0);
-	m_objects.resize(0);
-	m_objectsId.resize(0);
+	m_state->m_unknownOLEs.resize(0);
+	m_state->m_idToObjectMap.clear();
 
 	if (!file || !file->isStructured()) return false;
 
-	unsigned numSubStreams = file->subStreamCount();
 	//
 	// we begin by grouping the Ole by their potential main id
 	//
 	std::multimap<int, WPSOLEParserInternal::OleDef> listsById;
 	std::vector<int> listIds;
+	unsigned numSubStreams = file->subStreamCount();
 	for (unsigned i = 0; i < numSubStreams; ++i)
 	{
 		char const *nm=file->subStreamName(i);
@@ -264,7 +340,7 @@ bool WPSOLEParser::parse(RVNGInputStreamPtr file)
 		// separated the directory and the name
 		//    MatOST/MatadorObject1/Ole10Native
 		//      -> dir="MatOST/MatadorObject1", base="Ole10Native"
-		std::string::size_type pos = name.find_last_of('/');
+		auto pos = name.find_last_of('/');
 
 		std::string dir, base;
 		if (pos == std::string::npos) base = name;
@@ -282,54 +358,25 @@ bool WPSOLEParser::parse(RVNGInputStreamPtr file)
 #endif
 		WPSOLEParserInternal::OleDef data;
 		data.m_name = name;
-		data.m_dir = base;
-
-		// try to retrieve the identificator stored in the directory
-		//  MatOST/MatadorObject1/ -> 1, -1
-		//  Object 2/ -> 2, -1
-		dir+='/';
-		pos = dir.find('/');
-		int id[2] = { -1, -1};
-		while (pos != std::string::npos)
-		{
-			if (pos >= 1 && dir[pos-1] >= '0' && dir[pos-1] <= '9')
-			{
-				std::string::size_type idP = pos-1;
-				while (idP >=1 && dir[idP-1] >= '0' && dir[idP-1] <= '9')
-					idP--;
-				int val = std::atoi(dir.substr(idP, idP-pos).c_str());
-				if (id[0] == -1) id[0] = val;
-				else
-				{
-					id[1] = val;
-					break;
-				}
-			}
-			pos = dir.find('/', pos+1);
-		}
-		data.m_id = id[0];
-		data.m_subId = id[1];
+		data.m_base = base;
+		data.m_dir = dir;
+		data.m_id = m_state->m_directoryToIdFunction(dir);
 		if (listsById.find(data.m_id) == listsById.end())
 			listIds.push_back(data.m_id);
 		listsById.insert(std::multimap<int, WPSOLEParserInternal::OleDef>::value_type(data.m_id, data));
 	}
 
-	for (size_t i = 0; i < listIds.size(); i++)
+	for (auto id : listIds)
 	{
-		int id = listIds[i];
-
-		std::multimap<int, WPSOLEParserInternal::OleDef>::iterator pos =
-		    listsById.lower_bound(id);
+		auto pos = listsById.lower_bound(id);
 
 		// try to find a representation for each id
 		// FIXME: maybe we must also find some for each subid
-		WPSOLEParserObject pict;
-		int confidence = -1000;
-		WPSPosition potentialSize;
+		WPSEmbeddedObject pict;
 
 		while (pos != listsById.end())
 		{
-			WPSOLEParserInternal::OleDef const &dOle = pos->second;
+			auto const &dOle = pos->second;
 			if (pos->first != id) break;
 			++pos;
 
@@ -342,50 +389,21 @@ bool WPSOLEParser::parse(RVNGInputStreamPtr file)
 
 			libwps::DebugFile asciiFile(ole);
 			asciiFile.open(dOle.m_name);
-
-			librevenge::RVNGBinaryData data;
-			bool hasData = false;
-			int newConfidence = -2000;
-			std::string mime("image/pict");
 			bool ok = true;
-			WPSPosition pictPos;
 
 			try
 			{
-				if (readMM(ole, dOle.m_dir, asciiFile));
-				else if (readObjInfo(ole, dOle.m_dir, asciiFile));
-				else if (readOle(ole, dOle.m_dir, asciiFile));
-				else if (readMN0AndCheckWKS(ole, dOle.m_dir, data, asciiFile))
-				{
-					hasData = true;
-					newConfidence = 10;
-					mime="image/wks-ods";
-				}
-				else if (isOlePres(ole, dOle.m_dir) &&
-				         readOlePres(ole, data, pictPos, asciiFile))
-				{
-					hasData = true;
-					newConfidence = 2;
-				}
-				else if (isOle10Native(ole, dOle.m_dir) &&
-				         readOle10Native(ole, data, asciiFile))
-				{
-					hasData = true;
-					// small size can be a symptom that this is a link to a
-					// basic msworks data file, so we reduce confidence
-					newConfidence = data.size() > 1000 ? 4 : 2;
-				}
-				else if (readCompObj(ole, dOle.m_dir, asciiFile));
-				else if (readContents(ole, dOle.m_dir, data, pictPos, asciiFile))
-				{
-					hasData = true;
-					newConfidence = 3;
-				}
-				else if (readCONTENTS(ole, dOle.m_dir, data, pictPos, asciiFile))
-				{
-					hasData = true;
-					newConfidence = 3;
-				}
+				librevenge::RVNGPropertyList pList;
+				if (readMM(ole, dOle.m_base, asciiFile));
+				else if (readSummaryInformation(ole, dOle.m_base, dOle.m_dir.empty() ? m_state->m_metaData : pList, asciiFile));
+				else if (readObjInfo(ole, dOle.m_base, asciiFile));
+				else if (readOle(ole, dOle.m_base, asciiFile));
+				else if (readMN0AndCheckWKS(ole, dOle.m_base, pict, asciiFile));
+				else if (isOlePres(ole, dOle.m_base) && readOlePres(ole, pict, asciiFile));
+				else if (isOle10Native(ole, dOle.m_base) && readOle10Native(ole, pict, asciiFile));
+				else if (readCompObj(ole, dOle.m_base, asciiFile));
+				else if (readContents(ole, dOle.m_base, pict, asciiFile));
+				else if (readCONTENTS(ole, dOle.m_base, pict, asciiFile));
 				else
 					ok = false;
 			}
@@ -394,51 +412,12 @@ bool WPSOLEParser::parse(RVNGInputStreamPtr file)
 				ok = false;
 			}
 			if (!ok)
-			{
-				m_unknownOLEs.push_back(dOle.m_name);
-				asciiFile.reset();
-				continue;
-			}
-
-			if (hasData && data.size())
-			{
-				// probably only a subs data
-				if (dOle.m_subId != -1) newConfidence -= 10;
-
-				if (newConfidence > confidence ||
-				        (newConfidence == confidence && pict.m_data.size() < data.size()))
-				{
-					confidence = newConfidence;
-					pict.m_data = data;
-					pict.m_position = pictPos;
-					pict.m_mime = mime;
-				}
-
-				if (pict.m_position.naturalSize().x() > 0 && pict.m_position.naturalSize().y() > 0)
-					potentialSize = pict.m_position;
-#ifdef DEBUG_WITH_FILES
-				libwps::Debug::dumpFile(data, dOle.m_name.c_str());
-#endif
-			}
-
+				m_state->m_unknownOLEs.push_back(dOle.m_name);
 			asciiFile.reset();
-
-#ifndef DEBUG
-			if (confidence >= 3) break;
-#endif
 		}
 
-		if (pict.m_data.size())
-		{
-			m_objects.push_back(pict);
-			if (pict.m_position.naturalSize().x() <= 0. || pict.m_position.naturalSize().y() <= 0.)
-			{
-				Vec2f size = potentialSize.naturalSize();
-				if (size.x() > 0 && size.y() > 0)
-					pict.m_position.setNaturalSize(pict.m_position.getInvUnitScale(potentialSize.unit())*size);
-			}
-			m_objectsId.push_back(id);
-		}
+		if (!pict.isEmpty())
+			m_state->m_idToObjectMap[id]=pict;
 	}
 
 	return true;
@@ -463,10 +442,10 @@ bool WPSOLEParser::readOle(RVNGInputStreamPtr &ip, std::string const &oleName,
 	ip->seek(0, librevenge::RVNG_SEEK_SET);
 
 	int val[20];
-	for (int i= 0; i < 20; i++)
+	for (int &i : val)
 	{
-		val[i] = libwps::read8(ip);
-		if (val[i] < -10 || val[i] > 10) return false;
+		i = libwps::read8(ip);
+		if (i < -10 || i > 10) return false;
 	}
 
 	libwps::DebugStream f;
@@ -529,8 +508,8 @@ bool WPSOLEParser::readMM(RVNGInputStreamPtr &input, std::string const &oleName,
 	f << "@@MM:";
 
 	int val[6];
-	for (int i = 0; i < 6; i++)
-		val[i] = libwps::read16(input);
+	for (int &i : val)
+		i = libwps::read16(input);
 
 	switch (val[5])
 	{
@@ -577,7 +556,7 @@ bool WPSOLEParser::readCompObj(RVNGInputStreamPtr &ip, std::string const &oleNam
 
 	for (int i = 0; i < 6; i++)
 	{
-		long val = long(libwps::readU16(ip));
+		auto val = long(libwps::readU16(ip));
 		f << val << ", ";
 	}
 
@@ -587,15 +566,15 @@ bool WPSOLEParser::readCompObj(RVNGInputStreamPtr &ip, std::string const &oleNam
 	ascii.addPos(12);
 	// the clsid
 	unsigned long clsData[4]; // ushort n1, n2, n3, b8, ... b15
-	for (int i = 0; i < 4; i++)
-		clsData[i] = libwps::readU32(ip);
+	for (unsigned long &i : clsData)
+		i = libwps::readU32(ip);
 
 	f.str("");
 	f << "@@CompObj(CLSID):";
 	if (clsData[1] == 0 && clsData[2] == 0xC0 && clsData[3] == 0x46000000L)
 	{
 		// normally, a referenced object
-		char const *clsName = m_compObjIdName->getCLSName(clsData[0]);
+		char const *clsName = m_state->m_compObjIdName->getCLSName(clsData[0]);
 		if (clsName)
 			f << "'" << clsName << "'";
 		else
@@ -738,14 +717,10 @@ bool WPSOLEParser::isOlePres(RVNGInputStreamPtr &ip, std::string const &oleName)
 	return true;
 }
 
-bool WPSOLEParser::readOlePres(RVNGInputStreamPtr &ip, librevenge::RVNGBinaryData &data, WPSPosition &pos,
-                               libwps::DebugFile &ascii)
+bool WPSOLEParser::readOlePres(RVNGInputStreamPtr &ip, WPSEmbeddedObject &obj, libwps::DebugFile &ascii)
 {
-	data.clear();
 	if (!isOlePres(ip, "OlePres")) return false;
 
-	pos = WPSPosition();
-	pos.setUnit(librevenge::RVNG_POINT);
 	libwps::DebugStream f;
 	f << "@@OlePress(header): ";
 	ip->seek(0,librevenge::RVNG_SEEK_SET);
@@ -817,10 +792,10 @@ bool WPSOLEParser::readOlePres(RVNGInputStreamPtr &ip, librevenge::RVNGBinaryDat
 		f << val << ", ";
 	}
 	// dim in TWIP ?
-	long extendX = (long) libwps::readU32(ip);
-	long extendY = (long) libwps::readU32(ip);
-	if (extendX > 0 && extendY > 0)
-		pos.setNaturalSize(Vec2f(float(extendX)/20.f, float(extendY)/20.f));
+	auto extendX = (long) libwps::readU32(ip);
+	auto extendY = (long) libwps::readU32(ip);
+	if (extendX > 0 && extendY > 0 && obj.m_size!=Vec2f())
+		obj.m_size=Vec2f(float(extendX)/1440.f, float(extendY)/1440.f);
 	long fSize = libwps::read32(ip);
 	f << "extendX="<< extendX << ", extendY=" << extendY << ", fSize=" << fSize;
 
@@ -829,9 +804,15 @@ bool WPSOLEParser::readOlePres(RVNGInputStreamPtr &ip, librevenge::RVNGBinaryDat
 
 	if (fSize == 0) return ip->isEnd();
 
-	data.clear();
+	librevenge::RVNGBinaryData data;
 	if (!libwps::readData(ip, (unsigned long)fSize, data)) return false;
-
+	obj.add(data);
+#ifdef DEBUG_WITH_FILES
+	std::stringstream s;
+	static int num=0;
+	s << "OlePress" << num++;
+	libwps::Debug::dumpFile(data, s.str().c_str());
+#endif
 	if (!ip->isEnd())
 	{
 		ascii.addPos(ip->tell());
@@ -865,9 +846,7 @@ bool WPSOLEParser::isOle10Native(RVNGInputStreamPtr &ip, std::string const &oleN
 	return true;
 }
 
-bool WPSOLEParser::readOle10Native(RVNGInputStreamPtr &ip,
-                                   librevenge::RVNGBinaryData &data,
-                                   libwps::DebugFile &ascii)
+bool WPSOLEParser::readOle10Native(RVNGInputStreamPtr &ip, WPSEmbeddedObject &obj, libwps::DebugFile &ascii)
 {
 	if (!isOle10Native(ip, "Ole10Native")) return false;
 
@@ -880,8 +859,15 @@ bool WPSOLEParser::readOle10Native(RVNGInputStreamPtr &ip,
 	ascii.addPos(0);
 	ascii.addNote(f.str().c_str());
 
-	data.clear();
+	librevenge::RVNGBinaryData data;
 	if (!libwps::readData(ip, (unsigned long) fSize, data)) return false;
+	obj.add(data);
+#ifdef DEBUG_WITH_FILES
+	std::stringstream s;
+	static int num=0;
+	s << "Ole10_" << num++ << ".bmp";
+	libwps::Debug::dumpFile(data, s.str().c_str());
+#endif
 
 	if (!ip->isEnd())
 	{
@@ -906,15 +892,12 @@ bool WPSOLEParser::readOle10Native(RVNGInputStreamPtr &ip,
 ////////////////////////////////////////////////////////////////
 bool WPSOLEParser::readContents(RVNGInputStreamPtr &input,
                                 std::string const &oleName,
-                                librevenge::RVNGBinaryData &pict, WPSPosition &pos,
+                                WPSEmbeddedObject &obj,
                                 libwps::DebugFile &ascii)
 {
-	pict.clear();
 	if (strcmp(oleName.c_str(),"Contents") != 0) return false;
 
 	libwps::DebugStream f;
-	pos = WPSPosition();
-	pos.setUnit(librevenge::RVNG_POINT);
 	input->seek(0, librevenge::RVNG_SEEK_SET);
 	f << "@@Contents:";
 
@@ -934,7 +917,7 @@ bool WPSOLEParser::readContents(RVNGInputStreamPtr &input,
 	for (int i = 0; i < 3; i++)
 	{
 		/* 0,{10|21|75|101|116}x2 */
-		long val = (long) libwps::readU32(input);
+		auto val = (long) libwps::readU32(input);
 		if (val < 1000)
 			f << val << ",";
 		else
@@ -952,23 +935,8 @@ bool WPSOLEParser::readContents(RVNGInputStreamPtr &input,
 		WPS_DEBUG_MSG(("WPSOLEParser: warning: Contents header length\n"));
 		return false;
 	}
-	if (dim[0] > 0 && dim[0] < 3000 &&
-	        dim[1] > 0 && dim[1] < 3000)
-		pos.setSize(Vec2f(float(dim[0]),float(dim[1])));
-	else
-	{
-		WPS_DEBUG_MSG(("WPSOLEParser: warning: Contents odd size : %d %d\n", dim[0], dim[1]));
-	}
-	if (naturalSize[0] > 0 && naturalSize[0] < 5000 &&
-	        naturalSize[1] > 0 && naturalSize[1] < 5000)
-		pos.setNaturalSize(Vec2f(float(naturalSize[0]),float(naturalSize[1])));
-	else
-	{
-		WPS_DEBUG_MSG(("WPSOLEParser: warning: Contents odd naturalsize : %d %d\n", naturalSize[0], naturalSize[1]));
-	}
-
 	long actPos = input->tell();
-	long size = (long) libwps::readU32(input);
+	auto size = (long) libwps::readU32(input);
 	if (size <= 0) ok = false;
 	if (ok)
 	{
@@ -991,15 +959,40 @@ bool WPSOLEParser::readContents(RVNGInputStreamPtr &input,
 
 	if (ok)
 	{
-		if (libwps::readData(input,(unsigned long) size, pict))
+		librevenge::RVNGBinaryData data;
+		if (libwps::readData(input,(unsigned long) size, data))
+		{
+			obj.add(data);
 			ascii.skipZone(actPos+4, actPos+size+4-1);
+#ifdef DEBUG_WITH_FILES
+			std::stringstream s;
+			static int fileId=0;
+			s << oleName << ++fileId << "cntents.pict";
+			libwps::Debug::dumpFile(data, s.str().c_str());
+#endif
+		}
 		else
 		{
 			input->seek(actPos+4, librevenge::RVNG_SEEK_SET);
 			ok = false;
 		}
 	}
-
+	if (ok)
+	{
+		if (dim[0] > 0 && dim[0] < 3000 && dim[1] > 0 && dim[1] < 3000 && obj.m_size!=Vec2f())
+			obj.m_size=Vec2f(float(dim[0])/72.f,float(dim[1])/72.f);
+		else
+		{
+			WPS_DEBUG_MSG(("WPSOLEParser: warning: Contents odd size : %d %d\n", dim[0], dim[1]));
+		}
+		if (naturalSize[0] > 0 && naturalSize[0] < 5000 &&
+		        naturalSize[1] > 0 && naturalSize[1] < 5000 && obj.m_size!=Vec2f())
+			obj.m_size=Vec2f(float(dim[0])/72.f,float(dim[1])/72.f);
+		else
+		{
+			WPS_DEBUG_MSG(("WPSOLEParser: warning: Contents odd naturalsize : %d %d\n", naturalSize[0], naturalSize[1]));
+		}
+	}
 	if (!input->isEnd())
 	{
 		ascii.addPos(actPos);
@@ -1021,20 +1014,17 @@ bool WPSOLEParser::readContents(RVNGInputStreamPtr &input,
 ////////////////////////////////////////////////////////////////
 bool WPSOLEParser::readCONTENTS(RVNGInputStreamPtr &input,
                                 std::string const &oleName,
-                                librevenge::RVNGBinaryData &pict, WPSPosition &pos,
+                                WPSEmbeddedObject &obj,
                                 libwps::DebugFile &ascii)
 {
-	pict.clear();
 	if (strcmp(oleName.c_str(),"CONTENTS") != 0) return false;
 
 	libwps::DebugStream f;
 
-	pos = WPSPosition();
-	pos.setUnit(librevenge::RVNG_POINT);
 	input->seek(0, librevenge::RVNG_SEEK_SET);
 	f << "@@CONTENTS:";
 
-	long hSize = (long) libwps::readU32(input);
+	auto hSize = (long) libwps::readU32(input);
 	if (input->isEnd()) return false;
 	f << "hSize=" << std::hex << hSize << std::dec;
 
@@ -1048,9 +1038,9 @@ bool WPSOLEParser::readCONTENTS(RVNGInputStreamPtr &input,
 
 	// minimal checking of the "copied" header
 	input->seek(4, librevenge::RVNG_SEEK_SET);
-	long type = (long) libwps::readU32(input);
+	auto type = (long) libwps::readU32(input);
 	if (type < 0 || type > 4) return false;
-	long newSize = (long) libwps::readU32(input);
+	auto newSize = (long) libwps::readU32(input);
 
 	f << ", type=" << type;
 	if (newSize < 8) return false;
@@ -1063,10 +1053,11 @@ bool WPSOLEParser::readCONTENTS(RVNGInputStreamPtr &input,
 	for (int st = 0; st < 2 ; st++)
 	{
 		int dim[4];
-		for (int i = 0; i < 4; i++) dim[i] = libwps::read32(input);
+		for (int &i : dim) i = libwps::read32(input);
 
 		bool ok = dim[0] >= 0 && dim[2] > dim[0] && dim[1] >= 0 && dim[3] > dim[2];
-		if (ok && st==0) pos.setNaturalSize(Vec2f(float(dim[2]-dim[0]), float(dim[3]-dim[1])));
+		if (ok && st==0 && obj.m_size==Vec2f())
+			obj.m_size=Vec2f(float(dim[2]-dim[0])/72.f, float(dim[3]-dim[1])/72.f);
 		if (st==0) f << ", bdbox(Text)";
 		else f << ", bdbox(Data)";
 		if (!ok) f << "###";
@@ -1082,7 +1073,7 @@ bool WPSOLEParser::readCONTENTS(RVNGInputStreamPtr &input,
 		int val = libwps::readU16(input);
 		if (val) f << ",id"<< i << "=" << val;
 	}
-	long dataLength = (long) libwps::readU32(input);
+	auto dataLength = (long) libwps::readU32(input);
 	f << ",length=" << dataLength+hSize;
 
 	ascii.addPos(0);
@@ -1097,7 +1088,7 @@ bool WPSOLEParser::readCONTENTS(RVNGInputStreamPtr &input,
 		// or f0=a,f1=1,f2=2,f3=6c,f5=480,f6=360,f7=140,f8=f0
 		// or f0=61,f1=1,f2=2,f3=58,f5=280,f6=1e0,f7=a9,f8=7f
 		// f3=some header sub size ? f5/f6 and f7/f8 two other bdbox ?
-		long val = (long) libwps::readU32(input);
+		auto val = (long) libwps::readU32(input);
 		if (val) f << std::hex << ",f" << i << "=" << val;
 	}
 	for (int i = 0; 2*i+100 < hSize; i++)
@@ -1119,7 +1110,15 @@ bool WPSOLEParser::readCONTENTS(RVNGInputStreamPtr &input,
 	}
 
 	input->seek(4+hSize, librevenge::RVNG_SEEK_SET);
-	if (!libwps::readDataToEnd(input, pict)) return false;
+	librevenge::RVNGBinaryData data;
+	if (!libwps::readDataToEnd(input, data)) return false;
+	obj.add(data);
+#ifdef DEBUG_WITH_FILES
+	std::stringstream s;
+	static int fileId=0;
+	s << oleName << ++fileId << "Contents.pict";
+	libwps::Debug::dumpFile(data, s.str().c_str());
+#endif
 
 	ascii.skipZone(hSize+4, input->tell()-1);
 	return true;
@@ -1127,21 +1126,358 @@ bool WPSOLEParser::readCONTENTS(RVNGInputStreamPtr &input,
 
 ////////////////////////////////////////////////////////////////
 //
-// finally the MN0 subdirectory
+// the MN0 subdirectory
 //
 ////////////////////////////////////////////////////////////////
 bool WPSOLEParser::readMN0AndCheckWKS(RVNGInputStreamPtr &input, std::string const &oleName,
-                                      librevenge::RVNGBinaryData &data,  libwps::DebugFile &/*ascii*/)
+                                      WPSEmbeddedObject &obj, libwps::DebugFile &/*ascii*/)
 {
 	if (strcmp(oleName.c_str(),"MN0") != 0) return false;
-	WPSHeader *header=WPSHeader::constructHeader(input);
+	std::unique_ptr<WPSHeader> header(WPSHeader::constructHeader(input));
 	if (!header) return false;
 	bool ok=header->getKind()==WPS_SPREADSHEET;
-	delete header;
 	if (!ok) return false;
 
 	input->seek(0, librevenge::RVNG_SEEK_SET);
-	return libwps::readDataToEnd(input, data);
+	librevenge::RVNGBinaryData data;
+	if (!libwps::readDataToEnd(input, data))
+		return false;
+	obj.add(data,"image/wks-ods");
+#ifdef DEBUG_WITH_FILES
+	std::stringstream s;
+	static int fileId=0;
+	s << oleName << ++fileId << ".wks";
+	libwps::Debug::dumpFile(data, s.str().c_str());
+#endif
+	return true;
 }
 
+////////////////////////////////////////////////////////////
+//
+// Summary function
+//
+////////////////////////////////////////////////////////////
+bool WPSOLEParser::readSummaryInformation(RVNGInputStreamPtr input, std::string const &oleName,
+                                          librevenge::RVNGPropertyList &pList, libwps::DebugFile &ascii) const
+{
+	if (oleName!="SummaryInformation") return false;
+	input->seek(0, librevenge::RVNG_SEEK_END);
+	long endPos=input->tell();
+	input->seek(0, librevenge::RVNG_SEEK_SET);
+	libwps::DebugStream f;
+	f << "Entries(SumInfo):";
+	auto val=int(libwps::readU16(input));
+	if (endPos<48 || val!=0xfffe)
+	{
+		WPS_DEBUG_MSG(("WPSOLEParser::readSummaryInformation: header seems bad\n"));
+		f << "###";
+		ascii.addPos(0);
+		ascii.addNote(f.str().c_str());
+		return true;
+	}
+	for (int i=0; i<11; ++i)   // f1=1, f2=0-2
+	{
+		val=int(libwps::readU16(input));
+		if (val) f << "f" << i << "=" << val << ",";
+	}
+	unsigned long lVal=libwps::readU32(input);
+	if (lVal==0 || lVal>15)   // find 1 or 2 sections, unsure about the maximum numbers
+	{
+		WPS_DEBUG_MSG(("WPSOLEParser::readSummaryInformation: summary info is bad\n"));
+		f << "###sumInfo=" << std::hex << lVal << std::dec << ",";
+		ascii.addPos(0);
+		ascii.addNote(f.str().c_str());
+		return true;
+	}
+	auto numSection=int(lVal);
+	if (numSection!=1)
+		f << "num[section]=" << numSection << ",";
+	for (int i=0; i<4; ++i)
+	{
+		val=int(libwps::readU32(input));
+		static int const expected[]= {int(0xf29f85e0),0x10684ff9,0x891ab,int(0xd9b3272b)};
+		if (val==expected[i]) continue;
+		f << "#fmid" << i << "=" << std::hex << val << std::dec << ",";
+		static bool first=true;
+		if (first)
+		{
+			WPS_DEBUG_MSG(("WPSOLEParser::readSummaryInformation: fmid is bad\n"));
+			first=false;
+		}
+	}
+	auto decal=int(libwps::readU32(input));
+	if (decal<0x30 || endPos<decal)
+	{
+		WPS_DEBUG_MSG(("WPSOLEParser::readSummaryInformation: decal is bad\n"));
+		f << "decal=" << val << ",";
+		ascii.addPos(0);
+		ascii.addNote(f.str().c_str());
+		return true;
+
+	}
+	ascii.addPos(0);
+	ascii.addNote(f.str().c_str());
+	if (decal!=0x30)
+	{
+		ascii.addPos(0x30);
+		ascii.addNote("_");
+		input->seek(decal, librevenge::RVNG_SEEK_SET);
+	}
+
+	for (int sect=0; sect<numSection; ++sect)
+	{
+		long pos=input->tell();
+		f.str("");
+		f << "SumInfo-A:";
+		auto pSectSize=long(libwps::readU32(input));
+		long endSect=pos+pSectSize;
+		auto N=int(libwps::readU32(input));
+		f << "N=" << N << ",";
+		if (pSectSize<0 || endPos-pos<pSectSize || (pSectSize-8)/8<N)
+		{
+			WPS_DEBUG_MSG(("WPSOLEParser::readSummaryInformation: psetstruct is bad\n"));
+			f << "###";
+			ascii.addPos(pos);
+			ascii.addNote(f.str().c_str());
+			return true;
+		}
+		f << "[";
+		std::map<long,int> posToTypeMap;
+		for (int i=0; i<N; ++i)
+		{
+			auto type=int(libwps::readU32(input));
+			auto depl=int(libwps::readU32(input));
+			if (depl<=0) continue;
+			f << std::hex << depl << std::dec << ":" << type << ",";
+			if ((depl-8)/8<N || depl>pSectSize-4 || posToTypeMap.find(pos+depl)!=posToTypeMap.end())
+			{
+				f << "###";
+				continue;
+			}
+			posToTypeMap[pos+depl]=type;
+		}
+		f << "],";
+		ascii.addPos(pos);
+		ascii.addNote(f.str().c_str());
+
+		for (auto it=posToTypeMap.begin(); it!=posToTypeMap.end(); ++it)
+		{
+			pos=it->first;
+			auto nextIt=it;
+			if (++nextIt!=posToTypeMap.end()) endPos=nextIt->first;
+			input->seek(pos, librevenge::RVNG_SEEK_SET);
+			f.str("");
+			f << "SumInfo-B" << it->second << ":";
+			auto type=int(libwps::readU32(input));
+			if (sect==0 && it->second==1 && type==2)
+			{
+				long value=-1;
+				if (readSummaryPropertyLong(input,endPos,type,value,f) && value>=0 && value<10000) // 10000 is mac
+					f << "encoding,"; // USEME: never seems actually
+			}
+			else if (sect==0 && type==0x1e && ((it->second>=2 && it->second<=6) || it->second==8))
+			{
+				librevenge::RVNGString text;
+				if (readSummaryPropertyString(input, endPos, type, text, f) && !text.empty())
+				{
+					static char const *attribNames[] =
+					{
+						"", "", "dc:title", "dc:subject", "meta:initial-creator",
+						"meta:keywords", "dc:description"/*comment*/, "", "dc:creator"
+					};
+					pList.insert(attribNames[it->second], text);
+				}
+			}
+			else if (!readSummaryProperty(input, endPos, type, ascii, f))
+			{
+				WPS_DEBUG_MSG(("WPSOLEParser::readSummaryInformation: find unknown type\n"));
+				f << "##type=" << std::hex << type << std::dec << ",";
+			}
+			if (input->tell()!=endPos && input->tell()!=pos)
+				ascii.addDelimiter(input->tell(),'|');
+			ascii.addPos(pos);
+			ascii.addNote(f.str().c_str());
+		}
+		input->seek(endSect, librevenge::RVNG_SEEK_SET);
+	}
+	return true;
+}
+
+bool WPSOLEParser::readSummaryPropertyString(RVNGInputStreamPtr input, long endPos, int type,
+                                             librevenge::RVNGString &string, libwps::DebugStream &f) const
+{
+	if (!input) return false;
+	long pos=input->tell();
+	string.clear();
+	auto sSz=long(libwps::readU32(input));
+	if (sSz<0 || (endPos-pos-4)<sSz || pos+4+sSz>endPos)
+	{
+		WPS_DEBUG_MSG(("WPSOLEParser::readSummaryPropertyString: string size is bad\n"));
+		f << "##stringSz=" << sSz << ",";
+		return false;
+	}
+	std::string text("");
+	for (long c=0; c < sSz; ++c)
+	{
+		auto ch=char(libwps::readU8(input));
+		if (ch)
+		{
+			text+=ch;
+			libwps::appendUnicode
+			((uint32_t)libwps_tools_win::Font::unicode((unsigned char)ch,m_state->m_fontType), string);
+		}
+		else if (c+1!=sSz)
+			text+="##";
+	}
+	f << text;
+	if (type==0x1f && (sSz%4))
+		input->seek(sSz%4, librevenge::RVNG_SEEK_CUR);
+	return true;
+}
+
+bool WPSOLEParser::readSummaryPropertyLong(RVNGInputStreamPtr input, long endPos, int type, long &value,
+                                           libwps::DebugStream &f) const
+{
+	if (!input) return false;
+	long pos=input->tell();
+	switch (type)
+	{
+	case 2: // int
+	case 0x12: // uint
+		if (pos+2>endPos)
+			return false;
+		value=type==2 ? long(libwps::read16(input)) : long(libwps::readU16(input));
+		break;
+	case 3: // int
+	case 9: // uint
+		if (pos+4>endPos)
+			return false;
+		value=type==3 ? long(libwps::read32(input)) : long(libwps::readU32(input));
+		break;
+	default:
+		return false;
+	}
+	f << "val=" << value << ",";
+	return true;
+}
+
+bool WPSOLEParser::readSummaryProperty(RVNGInputStreamPtr input, long endPos, int type,
+                                       libwps::DebugFile &ascii, libwps::DebugStream &f) const
+{
+	if (!input) return false;
+	long pos=input->tell();
+	// see propread.cxx
+	if (type&0x1000)
+	{
+		auto N=int(libwps::readU32(input));
+		f << "N=" << N << ",";
+		f << "[";
+		for (int n=0; n<N; ++n)
+		{
+			pos=input->tell();
+			f << "[";
+			if (!readSummaryProperty(input, endPos, type&0xFFF, ascii, f))
+			{
+				input->seek(pos, librevenge::RVNG_SEEK_SET);
+				return false;
+			}
+			f << "],";
+		}
+		f << "],";
+		return true;
+	}
+	switch (type)
+	{
+	case 0x10: // int1
+	case 0x11: // uint1
+		if (pos+1>endPos)
+			return false;
+		f << "val=" << char(libwps::readU8(input));
+		break;
+	case 2: // int
+	case 0xb: // bool
+	case 0x12: // uint
+		if (pos+2>endPos)
+			return false;
+		if (type==2)
+			f << "val=" << int(libwps::read16(input)) << ",";
+		else if (type==0x12)
+			f << "val=" << int(libwps::readU16(input)) << ",";
+		else if (libwps::readU16(input))
+			f << "true,";
+		break;
+	case 3: // int
+	case 4: // float
+	case 9: // uint
+		if (pos+4>endPos)
+			return false;
+		if (type==3)
+			f << "val=" << int(libwps::read32(input)) << ",";
+		else if (type==9)
+			f << "val=" << int(libwps::readU32(input)) << ",";
+		else
+			f << "val[fl4]=" << std::hex << libwps::readU32(input) << std::dec << ",";
+		break;
+	case 5: // double
+	case 6:
+	case 7:
+	case 20:
+	case 21:
+	case 0x40:
+		if (pos+8>endPos)
+			return false;
+		ascii.addDelimiter(input->tell(),'|');
+		if (type==5)
+			f << "double,";
+		else if (type==6)
+			f << "cy,";
+		else if (type==7)
+			f << "date,";
+		else if (type==20)
+			f << "long,";
+		else if (type==21)
+			f << "ulong,";
+		else
+			f << "fileTime,"; // readme 8 byte
+		input->seek(pos+8, librevenge::RVNG_SEEK_SET);
+		break;
+	case 0xc: // variant
+		if (pos+4>endPos)
+			return false;
+		type=int(libwps::readU32(input));
+		return readSummaryProperty(input, endPos, type, ascii, f);
+	// case 20: int64
+	// case 21: uint64
+	case 8:
+	case 0x1e:
+	case 0x1f:
+	{
+		librevenge::RVNGString string;
+		if (!readSummaryPropertyString(input, endPos, type, string, f))
+			return false;
+		break;
+	}
+	case 0x41:
+	case 0x46:
+	case 0x47:
+	{
+		if (pos+4>endPos)
+			return false;
+		f << (type==0x41 ? "blob" : type==0x46 ? "blob[object]" : "clipboard") << ",";
+		auto dSz=long(libwps::readU32(input));
+		if (dSz<0 || pos+4+dSz>endPos)
+			return false;
+		if (dSz)
+		{
+			ascii.skipZone(pos+4, pos+4+dSz-1);
+			input->seek(dSz, librevenge::RVNG_SEEK_CUR);
+		}
+		break;
+	}
+	/* todo type==0x47, vtcf clipboard */
+	default:
+		return false;
+	}
+	return true;
+}
 /* vim:set shiftwidth=4 softtabstop=4 noexpandtab: */
