@@ -6,7 +6,10 @@
 
 #include "xfa/fxfa/cxfa_ffarc.h"
 
-CXFA_FFArc::CXFA_FFArc(CXFA_WidgetAcc* pDataAcc) : CXFA_FFDraw(pDataAcc) {}
+#include "xfa/fxfa/parser/cxfa_arc.h"
+#include "xfa/fxfa/parser/cxfa_value.h"
+
+CXFA_FFArc::CXFA_FFArc(CXFA_Node* pNode) : CXFA_FFWidget(pNode) {}
 
 CXFA_FFArc::~CXFA_FFArc() {}
 
@@ -16,17 +19,16 @@ void CXFA_FFArc::RenderWidget(CXFA_Graphics* pGS,
   if (!IsMatchVisibleStatus(dwStatus))
     return;
 
-  CXFA_Value value = m_pDataAcc->GetFormValue();
+  CXFA_Value* value = m_pNode->GetFormValueIfExists();
   if (!value)
     return;
 
-  CXFA_Arc arcObj = value.GetArc();
+  CFX_RectF rtArc = GetRectWithoutRotate();
+  CXFA_Margin* margin = m_pNode->GetMarginIfExists();
+  XFA_RectWithoutMargin(&rtArc, margin);
+
   CFX_Matrix mtRotate = GetRotateMatrix();
   mtRotate.Concat(matrix);
 
-  CFX_RectF rtArc = GetRectWithoutRotate();
-  if (CXFA_Margin mgWidget = m_pDataAcc->GetMargin())
-    XFA_RectWidthoutMargin(rtArc, mgWidget);
-
-  DrawBorder(pGS, arcObj, rtArc, mtRotate);
+  DrawBorder(pGS, value->GetArcIfExists(), rtArc, mtRotate);
 }
