@@ -6,6 +6,7 @@
 
 #include "xfa/fxfa/parser/cxfa_contentlayoutitem.h"
 
+#include "fxjs/xfa/cjx_object.h"
 #include "xfa/fxfa/parser/cxfa_node.h"
 
 CXFA_ContentLayoutItem::CXFA_ContentLayoutItem(CXFA_Node* pNode)
@@ -15,6 +16,6 @@ CXFA_ContentLayoutItem::CXFA_ContentLayoutItem(CXFA_Node* pNode)
       m_dwStatus(0) {}
 
 CXFA_ContentLayoutItem::~CXFA_ContentLayoutItem() {
-  if (m_pFormNode->GetUserData(XFA_LAYOUTITEMKEY) == this)
-    m_pFormNode->SetUserData(XFA_LAYOUTITEMKEY, nullptr);
+  if (m_pFormNode->JSObject()->GetLayoutItem() == this)
+    m_pFormNode->JSObject()->SetLayoutItem(nullptr);
 }
